package ca.recrown.islandsurvivalcraft.caching;

import java.util.HashMap;
import java.util.PriorityQueue;

public class Cache <CacheValueType> {
    private final int maxCacheSize;
    private final HashMap<Identifier, CacheValue<CacheValueType>> dataCache;
    private final PriorityQueue<Identifier> ids;
    private final float freshImportanceFactor;

    public Cache(int maxCacheSize, float freshImportanceFactor) {
        dataCache = new HashMap<>(maxCacheSize);
        ids = new PriorityQueue<>(maxCacheSize);
        this.maxCacheSize = maxCacheSize;
        this.freshImportanceFactor = freshImportanceFactor;
    }

    public Cache() {
        this(1024, 1f);
    }

    public Cache(int maxCacheSize) {
        this(maxCacheSize, 1f);
    }

    public synchronized CacheValue<CacheValueType> retrieveCache(Identifier identifier) {        
        if (dataCache.containsKey(identifier)) {
            ids.remove(identifier);
            CacheValue<CacheValueType> val = dataCache.get(identifier);
            val.identifier.usage++;
            ids.add(val.identifier);
            return dataCache.get(identifier);
        }
        if (ids.size() >= maxCacheSize) {
            dataCache.remove(ids.poll());
        }
        CacheValue<CacheValueType> value = new CacheValue<>(identifier);
        if (!ids.isEmpty()) {
            identifier.usage += ids.peek().usage * freshImportanceFactor;
        }
        ids.add(identifier);
        dataCache.put(identifier, value);

        return value;
    }

    public void clearCache() {
        ids.clear();
        dataCache.clear();
    }
}